package com.rshk.demo.services;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.rshk.demo.beans.Customer;
import com.rshk.demo.beans.CustomerGetRespData;
import com.rshk.demo.beans.CustomerListData;
import com.rshk.demo.beans.SaveCustomerRespData;

@Service
public class CustomerServiceImpl implements CustomerService {

	@Value("${customer_mcs}")
	private String mcsUrl;

	private final static Logger logger = LoggerFactory.getLogger(CustomerServiceImpl.class);

	public SaveCustomerRespData insert(Customer customer) throws Exception {

		try {
			HttpClient client = HttpClientBuilder.create().build();

			HttpPost request = new HttpPost(mcsUrl.concat("/save"));
			logger.info("We're going to call -> " + request.getURI().toString());
			request.setEntity(new StringEntity(new ObjectMapper().writeValueAsString(customer)));

			request.setHeader("Accept", "application/json");

			request.setHeader("Content-type", "application/json");
			logger.info("Request -> " + request);
			HttpResponse response = client.execute(request);
			String responseString = EntityUtils.toString(response.getEntity());
			logger.info("Getting response -> " + responseString);
			SaveCustomerRespData resp = new ObjectMapper().readValue(responseString, SaveCustomerRespData.class);
			return resp;

		} catch (Exception e) {
			logger.error("Error while trying to get customers ->", e);
			e.printStackTrace();
			throw e;
		}
	}

	public CustomerGetRespData get(String document) throws Exception {
		try {

			HttpClient client = HttpClientBuilder.create().build();

			HttpGet request = new HttpGet(mcsUrl.concat(String.format("/%s", document)));
			logger.info("We're going to call -> " + request.getURI().toString());
			HttpResponse response = client.execute(request);
			String responseString = EntityUtils.toString(response.getEntity());

			logger.info("Getting response -> " + responseString);

			/// deserialize response body into an object
			CustomerGetRespData data = new ObjectMapper().readValue(responseString, CustomerGetRespData.class);

			return data;

		} catch (Exception e) {
			logger.error("Error while trying to get customer ->", e);
			e.printStackTrace();
			throw e;
		}
	}

	public CustomerListData getCustomers() throws Exception {

		try {
			HttpClient client = HttpClientBuilder.create().build();

			HttpGet request = new HttpGet(mcsUrl.concat("/list"));
			logger.info("We're going to call -> " + request.getURI().toString());

			HttpResponse response = client.execute(request);
			String responseString = EntityUtils.toString(response.getEntity());
			logger.info("Getting response -> " + responseString);

			CustomerListData data = new ObjectMapper().readValue(responseString, CustomerListData.class);
			return data;

		} catch (Exception e) {
			logger.error("Error while trying to get customers ->", e);
			e.printStackTrace();
			throw e;
		}

	}

}
